@@ -129,7 +129,8 @@ typedef struct
     GList *pending_channels;
     
     TelepathyConnectionStatusReason abort_reason;
-    gboolean got_capabilities;
+    gboolean got_capabilities : 1;
+    gboolean setting_avatar : 1;
 
     gchar *alias;
 
@@ -215,8 +216,6 @@ static void mcd_async_request_chan_callback (DBusGProxy *proxy,
 static GError * map_tp_error_to_mc_error (McdChannel *channel, GError *tp_error);
 static void _mcd_connection_setup (McdConnection * connection);
 static void _mcd_connection_release_tp_connection (McdConnection *connection);
-static void request_avatar_cb (DBusGProxy *, GArray *, char *, GError *,
-			       gpointer);
 
 static McPresence presence_str_to_enum (const gchar *presence_str)
 {
@@ -791,6 +790,8 @@ static void
 set_avatar_cb (DBusGProxy *proxy, char *token, GError *error, gpointer userdata)
 {
     McdConnectionPrivate *priv = (McdConnectionPrivate *)userdata;
+
+    priv->setting_avatar = FALSE;
     if (error)
     {
 	g_warning ("%s: error: %s", G_STRFUNC, error->message);
@@ -819,29 +820,80 @@ clear_avatar_cb (DBusGProxy *proxy, GError *error, gpointer userdata)
     g_free (filename);
 }
 
+static void
+on_avatar_retrieved (DBusGProxy *proxy,
+		     guint       contact_id,
+		     gchar      *token,
+		     GArray     *avatar,
+		     gchar      *mime_type,
+		     gpointer    userdata)
+{
+    McdConnectionPrivate *priv = (McdConnectionPrivate *)userdata;
+    gchar *prev_token = NULL;
+    gchar *filename;
+
+#ifndef NO_AVATAR_UPDATED
+    if (contact_id != priv->self_handle) return;
+#endif
+    /* if we are setting the avatar, we must ignore this signal */
+    if (priv->setting_avatar) return;
+
+    g_debug ("%s: Avatar retrieved for contact %d, token: %s", G_STRFUNC, contact_id, token);
+    mc_account_get_avatar (priv->account, NULL, NULL, &prev_token);
+
+    if (!prev_token || strcmp (token, prev_token) != 0)
+    {
+	g_debug ("%s: received mime-type: %s", G_STRFUNC, mime_type);
+	if (mc_account_get_avatar (priv->account, &filename, NULL, NULL))
+	{
+	    g_file_set_contents (filename, avatar->data, avatar->len, NULL);
+	    mc_account_set_avatar_mime_type (priv->account, mime_type);
+	    mc_account_reset_avatar_id (priv->account);
+	    mc_account_set_avatar_token (priv->account, token);
+	    g_free (filename);
+	}
+    }
+    g_free (prev_token);
+}
+
+static void
+request_avatars_cb (DBusGProxy *proxy, GError *error, gpointer userdata)
+{
+    if (error)
+    {
+	g_warning ("%s: error: %s", G_STRFUNC, error->message);
+	g_error_free (error);
+    }
+}
+
 static gboolean
 on_avatar_updated (DBusGProxy *proxy, guint contact_id, const gchar *token,
 		   gpointer userdata)
 {
     McdConnectionPrivate *priv = (McdConnectionPrivate *)userdata;
-    gboolean changed = FALSE;
     gchar *prev_token;
+    gboolean changed = FALSE;
 
 #ifndef NO_AVATAR_UPDATED
     if (contact_id != priv->self_handle) return FALSE;
 #endif
+    /* if we are setting the avatar, we must ignore this signal */
+    if (priv->setting_avatar) return FALSE;
+
     g_debug ("%s: contact %d, token: %s", G_STRFUNC, contact_id, token);
     if (!mc_account_get_avatar (priv->account, NULL, NULL, &prev_token))
 	return FALSE;
 
     if (!prev_token || strcmp (token, prev_token) != 0)
     {
+    	GArray handles;
 	g_debug ("%s: avatar has changed", G_STRFUNC);
 	/* the avatar has changed, let's retrieve the new one */
-	tp_conn_iface_avatars_request_avatar_async (proxy, contact_id,
-						request_avatar_cb,
-						priv);
-	mc_account_set_avatar_token (priv->account, token);
+	handles.len = 1;
+	handles.data = (gchar *)&contact_id;
+	tp_conn_iface_avatars_request_avatars_async (proxy, &handles,
+						     request_avatars_cb,
+						     priv);
 	changed = TRUE;
     }
     g_free (prev_token);
@@ -856,6 +908,7 @@ _mcd_connection_set_avatar (McdConnectionPrivate *priv, gchar *filename,
     gchar *data = NULL;
     size_t length;
 
+    g_debug ("%s called", G_STRFUNC);
     if (filename == NULL || !g_file_test (filename, G_FILE_TEST_EXISTS))
     {
 	return;
@@ -870,6 +923,7 @@ _mcd_connection_set_avatar (McdConnectionPrivate *priv, gchar *filename,
 	    avatar.len = (guint)length;
 	    tp_conn_iface_avatars_set_avatar_async (priv->avatars_proxy,
 						    &avatar, mime_type, set_avatar_cb, priv);
+	    priv->setting_avatar = TRUE;
 	}
 	else
 	    tp_conn_iface_avatars_clear_avatar_async(priv->avatars_proxy,
@@ -921,21 +975,31 @@ static void
 _mcd_connection_setup_avatar (McdConnectionPrivate *priv)
 {
     gchar *filename, *mime_type, *token;
+    gboolean just_connected = FALSE;
 
-    priv->avatars_proxy = tp_conn_get_interface (priv->tp_conn,
-				    TELEPATHY_CONN_IFACE_AVATARS_QUARK);
     if (!priv->avatars_proxy)
     {
-	g_debug ("%s: connection does not support avatar interface", G_STRFUNC);
-	return;
-    }
-    g_object_ref (priv->avatars_proxy);
+	priv->avatars_proxy = tp_conn_get_interface (priv->tp_conn,
+					TELEPATHY_CONN_IFACE_AVATARS_QUARK);
+	if (!priv->avatars_proxy)
+	{
+	    g_debug ("%s: connection does not support avatar interface", G_STRFUNC);
+	    return;
+	}
+	g_object_ref (priv->avatars_proxy);
 #ifndef NO_AVATAR_UPDATED
-    dbus_g_proxy_connect_signal (priv->avatars_proxy,
-				 "AvatarUpdated",
-				 G_CALLBACK (on_avatar_updated),
-				 priv, NULL);
+	dbus_g_proxy_connect_signal (priv->avatars_proxy,
+				     "AvatarUpdated",
+				     G_CALLBACK (on_avatar_updated),
+				     priv, NULL);
+	dbus_g_proxy_connect_signal (priv->avatars_proxy,
+				     "AvatarRetrieved",
+				     G_CALLBACK (on_avatar_retrieved),
+				     priv, NULL);
 #endif
+	just_connected = TRUE;
+	priv->setting_avatar = FALSE;
+    }
     if (!mc_account_get_avatar (priv->account, &filename, &mime_type, &token))
     {
 	g_debug ("%s: mc_account_get_avatar() returned FALSE", G_STRFUNC);
@@ -946,10 +1010,11 @@ _mcd_connection_setup_avatar (McdConnectionPrivate *priv)
     {
 	if (!token)
 	    _mcd_connection_set_avatar (priv, filename, mime_type);
-	else
+	else if (just_connected)
 	{
 	    GArray handles;
 
+	    g_debug ("checking for server token");
 	    /* Set the avatar only if no other one was set */
 	    handles.len = 1;
 	    handles.data = (gchar *)&priv->self_handle;
@@ -1552,6 +1617,10 @@ _mcd_connection_release_tp_connection (McdConnection *connection)
 					"AvatarUpdated",
 					G_CALLBACK (on_avatar_updated),
 					priv);
+	dbus_g_proxy_disconnect_signal (priv->avatars_proxy,
+					"AvatarRetrieved",
+					G_CALLBACK (on_avatar_retrieved),
+					priv);
 #endif
 	g_object_unref (priv->avatars_proxy);
 	priv->avatars_proxy = NULL;
@@ -1955,9 +2024,13 @@ remove_capabilities_refs (gpointer data)
 static gint
 pending_channel_cmp (const McdPendingChannel *a, const McdPendingChannel *b)
 {
-    return a->handle == b->handle &&
-	a->handle_type == b->handle_type &&
-	strcmp (a->type, b->type);
+    gint ret;
+
+    ret = a->handle - b->handle;
+    if (ret) return ret;
+    ret = a->handle_type - b->handle_type;
+    if (ret) return ret;
+    return strcmp (a->type, b->type);
 }
 
 static void
@@ -2414,29 +2487,6 @@ mcd_connection_cancel_channel_request (McdConnection *connection,
     return FALSE;
 }
 
-static void
-request_avatar_cb (DBusGProxy *proxy, GArray *avatar, char *mime_type,
-		   GError *error, gpointer userdata)
-{
-    McdConnectionPrivate *priv = (McdConnectionPrivate *)userdata;
-    gchar *filename;
-    if (error)
-    {
-	g_warning ("%s: error: %s", G_STRFUNC, error->message);
-	g_error_free (error);
-	return;
-    }
-    g_debug ("%s: received mime-type: %s", G_STRFUNC, mime_type);
-    if (mc_account_get_avatar (priv->account, &filename, NULL, NULL))
-    {
-	g_file_set_contents (filename, avatar->data, avatar->len, NULL);
-	mc_account_set_avatar_mime_type (priv->account, mime_type);
-	mc_account_reset_avatar_id (priv->account);
-	g_free (filename);
-    }
-    g_free (mime_type);
-}
-
 /**
  * mcd_connection_remote_avatar_changed:
  * @connection: the #McdConnection.
