@@ -93,6 +93,18 @@ _mcd_manager_create_connection (McdManager * manager, McAccount * account)
     McdManagerPrivate *priv = MCD_MANAGER_PRIV (manager);
     
     g_return_if_fail (mcd_manager_get_account_connection (manager, account) == NULL);
+    if (!priv->tp_conn_mgr)
+    {
+	g_return_if_fail (MC_IS_MANAGER (priv->mc_manager));
+
+	priv->tp_conn_mgr =
+	    tp_connmgr_new (priv->dbus_connection,
+			    mc_manager_get_bus_name (priv->mc_manager),
+			    mc_manager_get_object_path (priv->mc_manager),
+			    TP_IFACE_CONN_MGR_INTERFACE);
+	g_debug ("%s: Manager %s created", G_STRFUNC,
+		 mc_manager_get_unique_name (priv->mc_manager));
+    }
     
     connection = mcd_connection_new (priv->dbus_connection,
             mc_manager_get_bus_name (priv->
@@ -113,8 +125,6 @@ _mcd_manager_create_connections (McdManager * manager)
     GList *node;
     McdManagerPrivate *priv = MCD_MANAGER_PRIV (manager);
 
-    g_return_if_fail (TELEPATHY_IS_CONNMGR (priv->tp_conn_mgr));
-
     for (node = priv->accounts; node; node = node->next)
     {
 	/* Create a connection object for each account */
@@ -248,22 +258,6 @@ on_presence_requested_idle (gpointer data)
 	&& (requested_presence != MC_PRESENCE_OFFLINE
 	    && requested_presence != MC_PRESENCE_UNSET))
     {
-	if (!priv->tp_conn_mgr)
-	{
-	    g_return_val_if_fail (MC_IS_MANAGER (priv->mc_manager),
-				  FALSE);
-
-	    priv->tp_conn_mgr =
-		tp_connmgr_new (priv->dbus_connection,
-				mc_manager_get_bus_name (priv->
-							    mc_manager),
-				mc_manager_get_object_path (priv->
-							       mc_manager),
-				TP_IFACE_CONN_MGR_INTERFACE);
-	    g_debug ("%s: Manager %s created", G_STRFUNC,
-		     mc_manager_get_unique_name (priv->mc_manager));
-	}
-
 	_mcd_manager_create_connections (manager);
     }
 
@@ -843,7 +837,7 @@ mcd_manager_add_account (McdManager * manager, McAccount * account)
 {
     McdManagerPrivate *priv;
     McdConnection *connection;
-    McPresence actual_presence;
+    McPresence requested_presence;
 
     g_return_val_if_fail (MCD_IS_MANAGER (manager), FALSE);
     g_return_val_if_fail (MC_IS_ACCOUNT (account), FALSE);
@@ -864,35 +858,17 @@ mcd_manager_add_account (McdManager * manager, McAccount * account)
     priv->accounts = g_list_prepend (priv->accounts, account);
     g_debug ("%s: %u accounts in total", G_STRFUNC, g_list_length (priv->accounts));
     
-    actual_presence =
-	mcd_presence_frame_get_actual_presence (priv->presence_frame);
+    requested_presence =
+	mcd_presence_frame_get_requested_presence (priv->presence_frame);
 
     connection = mcd_manager_get_account_connection (manager, account);
     if (!connection)
     {
         /* if presence is not offline or unset, we must create the 
          * connection for this new account */
-        if ((actual_presence != MC_PRESENCE_OFFLINE &&
-                    actual_presence != MC_PRESENCE_UNSET))
+        if ((requested_presence != MC_PRESENCE_OFFLINE &&
+             requested_presence != MC_PRESENCE_UNSET))
         {
-            /* Also create the telepathy connection manager if not already
-             * created */
-            if (!priv->tp_conn_mgr)
-            {
-                g_return_val_if_fail (MC_IS_MANAGER (priv->mc_manager),
-                        FALSE);
-
-                priv->tp_conn_mgr =
-                    tp_connmgr_new (priv->dbus_connection,
-                            mc_manager_get_bus_name (priv->
-                                mc_manager),
-                            mc_manager_get_object_path (priv->
-                                mc_manager),
-                            TP_IFACE_CONN_MGR_INTERFACE);
-                g_debug ("%s: Manager %s created", G_STRFUNC,
-                        mc_manager_get_unique_name (priv->mc_manager));
-            }
-
             _mcd_manager_create_connection (manager, account);
         }
     }
