@@ -72,7 +72,7 @@ typedef struct _McdMasterPrivate
     McdDispatcher *dispatcher;
     McdProxy *proxy;
     McPresence awake_presence;
-    const gchar *awake_presence_message;
+    gchar *awake_presence_message;
     McPresence default_presence;
 
     /* We create this for our member objects */
@@ -86,6 +86,8 @@ typedef struct _McdMasterPrivate
     gboolean offline_on_idle;
     GHashTable *clients_needing_presence;
 
+    GHashTable *extra_parameters;
+
     gboolean is_disposed;
 } McdMasterPrivate;
 
@@ -561,8 +563,10 @@ _mcd_master_disconnect (McdMission * mission)
 static void
 _mcd_master_finalize (GObject * object)
 {
-    McdMaster *master;
-    master = MCD_MASTER (object);
+    McdMasterPrivate *priv = MCD_MASTER_PRIV (object);
+
+    g_free (priv->awake_presence_message);
+
     G_OBJECT_CLASS (mcd_master_parent_class)->finalize (object);
 }
 
@@ -630,8 +634,10 @@ _mcd_master_set_flags (McdMission * mission, McdSystemFlags flags)
 		mcd_presence_frame_get_actual_presence (priv->presence_frame);
 	    if (priv->awake_presence != MC_PRESENCE_AVAILABLE)
 		return;
-	    priv->awake_presence_message =
-		mcd_presence_frame_get_actual_presence_message (priv->presence_frame);
+	    g_free (priv->awake_presence_message);
+	    priv->awake_presence_message = g_strdup
+		(mcd_presence_frame_get_actual_presence_message
+		 (priv->presence_frame));
 
 	    mcd_presence_frame_request_presence (priv->presence_frame,
 						 MC_PRESENCE_AWAY, NULL);
@@ -752,6 +758,14 @@ install_dbus_filter (McdMasterPrivate *priv)
     }
 }
 
+static void
+_g_value_free (gpointer data)
+{
+  GValue *value = (GValue *) data;
+  g_value_unset (value);
+  g_free (value);
+}
+
 static void
 mcd_master_init (McdMaster * master)
 {
@@ -786,6 +800,9 @@ mcd_master_init (McdMaster * master)
     priv->clients_needing_presence = g_hash_table_new_full (g_str_hash,
 							    g_str_equal,
 							    g_free, NULL);
+
+    priv->extra_parameters = g_hash_table_new_full (g_str_hash, g_str_equal,
+						    g_free, _g_value_free);
 }
 
 McdMaster *
@@ -1206,3 +1223,57 @@ mcd_master_set_default_presence_setting (McdMaster *master,
     priv->default_presence = presence;
 }
 
+/**
+ * mcd_master_add_connection_parameter:
+ * @master: the #McdMaster.
+ * @name: the name of the parameter to add.
+ * @value: a #GValue.
+ *
+ * Set a global connection parameter to be passed to all connection managers
+ * (which support this parameter).  If called twice for the same parameter, the
+ * new value will replace the previous one.
+ */
+void
+mcd_master_add_connection_parameter (McdMaster *master, const gchar *name,
+				     const GValue *value)
+{
+    McdMasterPrivate *priv = MCD_MASTER_PRIV (master);
+    GValue *val;
+
+    g_return_if_fail (name != NULL);
+    g_return_if_fail (value != NULL);
+
+    val = g_malloc0 (sizeof (GValue));
+    g_value_init (val, G_VALUE_TYPE (value));
+    g_value_copy (value, val);
+    g_hash_table_replace (priv->extra_parameters, g_strdup (name), val);
+}
+
+static void
+copy_parameter (gpointer key, gpointer value, gpointer userdata)
+{
+    GHashTable *dest = (GHashTable *)userdata;
+
+    g_hash_table_insert (dest, key, value);
+}
+
+/**
+ * mcd_master_get_connection_parameters:
+ * @master: the #McdMaster.
+ *
+ * Get the global connections parameters.
+ *
+ * Returns: the #GHashTable of the parameters. It has to be destroyed when no
+ * longer needed.
+ */
+GHashTable *
+mcd_master_get_connection_parameters (McdMaster *master)
+{
+    McdMasterPrivate *priv = MCD_MASTER_PRIV (master);
+    GHashTable *ret;
+    
+    ret = g_hash_table_new (g_str_hash, g_str_equal);
+    g_hash_table_foreach (priv->extra_parameters, copy_parameter, ret);
+    return ret;
+}
+
